@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 /* client.h
@@ -69,11 +70,11 @@ typedef struct _client_tag
 
 int client_create (client_t **c_ptr, connection_t *con, http_parser_t *parser);
 void client_destroy(client_t *client);
-void client_send_504(client_t *client, char *message);
-void client_send_404(client_t *client, char *message);
+void client_send_100(client_t *client);
+void client_send_404(client_t *client, const char *message);
 void client_send_401(client_t *client);
-void client_send_403(client_t *client, const char *reason);
-void client_send_400(client_t *client, char *message);
+void client_send_403(client_t *client, const char *message);
+void client_send_400(client_t *client, const char *message);
 int client_send_bytes (client_t *client, const void *buf, unsigned len);
 int client_read_bytes (client_t *client, void *buf, unsigned len);
 void client_set_queue (client_t *client, refbuf_t *refbuf);
