@@ -8,6 +8,7 @@
  *                      oddsock <oddsock@xiph.org>,
  *                      Karl Heyes <karl@xiph.org>
  *                      and others (see AUTHORS for details).
+ * Copyright 2011-2012, Philipp "ph3-der-loewe" Schafft <lion@lion.leolix.org>,
  */
 
 #ifdef HAVE_CONFIG_H
@@ -485,6 +486,100 @@ char *util_base64_decode(const char *data)
     return result;
 }
 
+ssize_t util_http_build_header(char * out, size_t len, ssize_t offset,
+        int cache,
+        int status, const char * statusmsg,
+        const char * contenttype, const char * charset,
+        const char * datablock) {
+    const char * http_version = "1.0";
+    ice_config_t *config;
+    time_t now;
+    struct tm result;
+    struct tm *gmtime_result;
+    char currenttime_buffer[80];
+    char status_buffer[80];
+    char contenttype_buffer[80];
+    ssize_t ret;
+
+    if (!out)
+        return -1;
+
+    if (offset == -1)
+        offset = strlen (out);
+
+    out += offset;
+    len -= offset;
+
+    if (status == -1)
+    {
+        status_buffer[0] = '\0';
+    }
+    else
+    {
+        if (!statusmsg)
+	{
+	    switch (status)
+	    {
+	        case 200: statusmsg = "OK"; break;
+		case 206: statusmsg = "Partial Content"; http_version = "1.1"; break;
+		case 400: statusmsg = "Bad Request"; break;
+		case 401: statusmsg = "Authentication Required"; break;
+		case 403: statusmsg = "Forbidden"; break;
+		case 404: statusmsg = "File Not Found"; break;
+		case 416: statusmsg = "Request Range Not Satisfiable"; break;
+		default:  statusmsg = "(unknown status code)"; break;
+	    }
+	}
+	snprintf (status_buffer, sizeof (status_buffer), "HTTP/%s %d %s\r\n", http_version, status, statusmsg);
+    }
+
+    if (contenttype)
+    {
+    	if (charset)
+            snprintf (contenttype_buffer, sizeof (contenttype_buffer), "Content-Type: %s; charset=%s\r\n",
+	                                                               contenttype, charset);
+	else
+            snprintf (contenttype_buffer, sizeof (contenttype_buffer), "Content-Type: %s\r\n",
+                                                                       contenttype);
+    }
+    else
+    {
+        contenttype_buffer[0] = '\0';
+    }
+
+    time(&now);
+#ifndef _WIN32
+    gmtime_result = gmtime_r(&now, &result);
+#else
+    /* gmtime() on W32 breaks POSIX and IS thread-safe (uses TLS) */
+    gmtime_result = gmtime (&now);
+    if (gmtime_result)
+        memcpy (&result, gmtime_result, sizeof (result));
+#endif
+
+    if (gmtime_result)
+        strftime(currenttime_buffer, sizeof(currenttime_buffer), "Date: %a, %d %b %Y %X GMT\r\n", gmtime_result);
+    else
+        currenttime_buffer[0] = '\0';
+
+    config = config_get_config();
+    ret = snprintf (out, len, "%sServer: %s\r\n%s%s%s%s%s%s",
+                              status_buffer,
+			      config->server_id,
+			      currenttime_buffer,
+			      contenttype_buffer,
+			      (status == 401 ? "WWW-Authenticate: Basic realm=\"Icecast2 Server\"\r\n" : ""),
+                              (cache     ? "" : "Cache-Control: no-cache\r\n"
+                                                "Expires: Mon, 26 Jul 1997 05:00:00 GMT\r\n"
+                                                "Pragma: no-cache\r\n"),
+                              (datablock ? "\r\n" : ""),
+                              (datablock ? datablock : ""));
+    config_release_config();
+
+    return ret;
+}
+
+
 util_dict *util_dict_new(void)
 {
     return (util_dict *)calloc(1, sizeof(util_dict));
