@@ -25,17 +25,15 @@
 #define  YP_CLUSTER_PASSWORD 9
 #define  YP_SUBTYPE 10 
 
-struct source_tag;
-
 #define YP_ADD_ALL -1
 
 #ifdef USE_YP
-void yp_add (struct source_tag *source);
+void yp_add (const char *mount);
 void yp_remove (const char *mount);
 void yp_touch (const char *mount);
 void yp_recheck_config (ice_config_t *config);
-void yp_initialize();
-void yp_shutdown();
+void yp_initialize(void);
+void yp_shutdown(void);
 
 #else
 
